/**
* @file memory.c
* @brief Memory helpers impl
* @author Kosenkov
* @date 06.04.2018
*/

#include "memory.h"

/**
* Write words to CCPU
* 
* @param addr Addres of CCPU mapped in memory for writing words
* @param data Data array to be written into CCPU
* @param sz Size of buffers in uint32s
* @author Kosenkov
* @date 10.01.2018
*/
void writeBy32(volatile uint32_t* addr, const uint32_t* data, Sz32 sz)
{
    uint32_t i;
    for (i = 0; i < sz; ++i)
    {
        addr[i] = data[i];
    }
}

/**
* Read words from CCPU
* 
* @param addr Addres of CCPU mapped in memory for reading words
* @param data Data array to be read from CCPU
* @param sz Size of buffers in uint32s
* @author Kosenkov
* @date 10.01.2018
*/
void readBy32(const volatile uint32_t* addr, uint32_t* data, Sz32 sz)
{
    uint32_t i;
    for (i = 0; i < sz; ++i)
    {
        data[i] = addr[i];
    }
}
