/**
* @file memory.h
* @brief Memory helpers header
* @author Kosenkov
* @date 06.04.2018
*/

#ifndef __MEMORY_H__
#define __MEMORY_H__

#include "types.h"

#include <string.h>

// wrappers
#define mldrMemSet(dst, byte, sz) memset(dst, byte, sz)
#define mldrMemCpy(dst, src, sz) memcpy(dst, src, sz)
#define mldrMemCmp(lhs, rhs, sz) memcmp(lhs, rhs, sz)

/**
* Write words to CCPU
* 
* @param addr Addres of CCPU mapped in memory for writing words
* @param data Data array to be written into CCPU
* @param sz Size of buffers in uint32s
* @author Kosenkov
* @date 10.01.2018
*/
void writeBy32(volatile uint32_t* addr, const uint32_t* data, Sz32 sz);

/**
* Read words from CCPU
* 
* @param addr Addres of CCPU mapped in memory for reading words
* @param data Data array to be read from CCPU
* @param sz Size of buffers in uint32s
* @author Kosenkov
* @date 10.01.2018
*/
void readBy32(const volatile uint32_t* addr, uint32_t* data, Sz32 sz);

#endif
