/**
* @file otpData.h
* @brief Defines and structures
* @author Kosenkov
* @date 25.05.2018
*/

#ifndef __OTPDATA_H__
#define __OTPDATA_H__

#include "types.h"

// OTP statuses
#define OTP_VIRGIN              ((uint32_t)0x00000000)
#define OTP_PREINIT             ((uint32_t)0x0000A55A)
#define OTP_INIT                ((uint32_t)0x75A9A55A)

#define SZ_OTP                  ((uint32_t)((64 * 1024) >> 2)) // 64KB in uint32_t units
#define SZ_OTP_BOOT_ROM         ((uint32_t)((16 * 1024) >> 2)) // 16KB in uint32_t units
#define SZ_OTP_USER             ((uint32_t)((48 * 1024) >> 2)) // 48KB in uint32_t units
compiletime_assert((SZ_OTP_USER + SZ_OTP_BOOT_ROM) == SZ_OTP);

#define SZ_CRC                  ((uint32_t)2)
#define SZ_SN                   ((uint32_t)2)

#define SZ_GAMMA_FULL           (SZ_OTP_BOOT_ROM - (sizeof(OtpServiceData) >> 2))

#ifndef SMALL_GAMMA
    #define SZ_GAMMA            SZ_GAMMA_FULL
#else
    #define SZ_GAMMA            ((uint32_t)4)
    #define SZ_PROG             ((uint32_t)600)
#endif

/**
* Structure of OTP user arrea
*/
typedef struct
{
    uint32_t    Special;        /*< Special word  */
    uint32_t    NotUsed[255];   /*< Testing arrea */
    uint32_t    User[12032];    /*< User space    */
} OtpUserData;
compiletime_assert((sizeof(OtpUserData) >> 2) == SZ_OTP_USER);

/**
* Boot ROM service space in OTP
*/
typedef struct
{
    uint32_t    Status;         /*< Current OTP status                              */
    uint32_t    Sn[SZ_SN];      /*< Serial number                                   */
    uint32_t    Crc[SZ_CRC];    /*< 2 crcs of OTP. First on preinit, second on init */
} OtpServiceData;
compiletime_assert((sizeof(OtpServiceData) >> 2) == (SZ_SN + SZ_CRC + 1));

/**
* OTP structure
*/
typedef struct
{
    OtpUserData     UData;              /*< User space    */
    OtpServiceData  SData;              /*< Service space */
    uint32_t        Gamma[SZ_GAMMA];    /*< Gamma arrea   */
} OtpData;

#ifndef SMALL_GAMMA
compiletime_assert((sizeof(OtpData) >> 2) == SZ_OTP);
#endif

#endif
