#ifndef __CM0_H__
#define __CM0_H__

/*
 * ==========================================================================
 * ---------- Interrupt Number Definition -----------------------------------
 * ==========================================================================
 */
typedef enum
{
    /******  Cortex-M0 Processor Exceptions Numbers ***********************************/
    NonMaskableInt_IRQn         = -14,      /*< 2 Non Maskable Interrupt             */
    HardFault_IRQn              = -13,      /*< 3 Cortex-M0 Hard Fault Interrupt     */
    SVC_IRQn                    = -5,       /*< 11 Cortex-M0 SV Call Interrupt       */
    PendSV_IRQn                 = -2,       /*< 14 Cortex-M0 Pend SV Interrupt       */
    SysTick_IRQn                = -1,       /*< 15 Cortex-M0 System Tick Interrupt   */

    /******  STM32F0 specific Interrupt Numbers ***************************************/
    Gate0Fifo_IRQn              = 0,        /*< Gate Channel 0 FIFO Interrupt        */
    Gate0Regs_IRQn              = 1,        /*< Gate Channel 0 Registers Interrupt   */
    Gate1Fifo_IRQn              = 4,        /*< Gate Channel 1 FIFO Interrupt        */
    Gate1Regs_IRQn              = 5,        /*< Gate Channel 1 Registers Interrupt   */
    Alu_IRQn                    = 8,        /*< ALU Interrupt                        */
    LBlock_IRQn                 = 12,       /*< LBlock Interrupt                     */
    GPIO_0_IRQn                 = 16,       /*< GPIO Interrupt pin 0                 */
    GPIO_1_IRQn                 = 17,       /*< GPIO Interrupt pin 1                 */
    GPIO_2_3_IRQn               = 18,       /*< GPIO Interrupt pins 2 - 3            */
    GPIO_0_3_IRQn               = 19,       /*< GPIO Interrupt pins 0 - 3            */
    GPIO_4_11_IRQn              = 20,       /*< GPIO Interrupt pins 4 - 11           */
    GPIO_0_11_IRQn              = 21,       /*< GPIO Interrupt pins 0 - 11           */
    UART_IRQn                   = 24,       /*< UART Interrupt                       */
    RND_IRQn                    = 28        /*< Random Sensor Interrupt              */
} IRQn_Type;

#define __NVIC_PRIO_BITS  2         /* size of NVIC Priority control            */
#include "core_cm0.h"               /* Cortex-M0 processor and core peripherals */
#include <stdint.h>                 /* Type with size declaration               */

/*
 * ==========================================================================
 * ------------------------ P Byte registers --------------------------------
 * ==========================================================================
 */
typedef struct
{
    __IO uint32_t TRM_00;           /*< Transformation register 0,         Address offset: 0x00 */
    __IO uint32_t TRM_01;           /*< Transformation register 1,         Address offset: 0x04 */
    __IO uint32_t TRM_02;           /*< Transformation register 2,         Address offset: 0x08 */
    __IO uint32_t TRM_03;           /*< Transformation register 3,         Address offset: 0x0C */
    __IO uint32_t TRM_04;           /*< Transformation register 4,         Address offset: 0x10 */
    __IO uint32_t TRM_05;           /*< Transformation register 5,         Address offset: 0x14 */
    __IO uint32_t TRM_06;           /*< Transformation register 6,         Address offset: 0x18 */
    __IO uint32_t TRM_07;           /*< Transformation register 7,         Address offset: 0x1C */
    __IO uint32_t TRM_08;           /*< Transformation register 8,         Address offset: 0x20 */
    __IO uint32_t TRM_09;           /*< Transformation register 9,         Address offset: 0x24 */
    __IO uint32_t TRM_10;           /*< Transformation register 10,        Address offset: 0x28 */
    __IO uint32_t TRM_11;           /*< Transformation register 11,        Address offset: 0x2C */
    __IO uint32_t TRM_12;           /*< Transformation register 12,        Address offset: 0x30 */
    __IO uint32_t TRM_13;           /*< Transformation register 13,        Address offset: 0x34 */
    __IO uint32_t TRM_14;           /*< Transformation register 14,        Address offset: 0x38 */
    __IO uint32_t TRM_15;           /*< Transformation register 15,        Address offset: 0x3C */
    __IO uint32_t MUX_SET;          /*< Transformation setup register,     Address offset: 0x40 */
    __IO uint32_t MUX_VAL;          /*< Transformation value register,     Address offset: 0x44 */
} PBYTE_TypeDef;

/*
 * ==========================================================================
 * ------------------------ P Bit registers ---------------------------------
 * ==========================================================================
 */
typedef struct
{
    __IO uint32_t TRM_00;           /*< Transformation register 0,         Address offset: 0x00 */
    __IO uint32_t TRM_01;           /*< Transformation register 1,         Address offset: 0x04 */
    __IO uint32_t TRM_02;           /*< Transformation register 2,         Address offset: 0x08 */
    __IO uint32_t TRM_03;           /*< Transformation register 3,         Address offset: 0x0C */
    __IO uint32_t MUX_SET;          /*< Transformation setup register,     Address offset: 0x10 */
    __IO uint32_t MUX_VAL;          /*< Transformation value register,     Address offset: 0x14 */
} PBIT_TypeDef;

/*
 * ==========================================================================
 * ------------------------ S Block registers -------------------------------
 * ==========================================================================
 */
typedef struct
{
    __IO uint32_t TRM_00;           /*< Transformation register 0,         Address offset: 0x00 */
    __IO uint32_t TRM_01;           /*< Transformation register 1,         Address offset: 0x04 */
    __IO uint32_t TRM_02;           /*< Transformation register 2,         Address offset: 0x08 */
    __IO uint32_t TRM_03;           /*< Transformation register 3,         Address offset: 0x0C */
    __IO uint32_t TRM_04;           /*< Transformation register 0,         Address offset: 0x10 */
    __IO uint32_t TRM_05;           /*< Transformation register 1,         Address offset: 0x14 */
    __IO uint32_t TRM_06;           /*< Transformation register 2,         Address offset: 0x18 */
    __IO uint32_t TRM_07;           /*< Transformation register 3,         Address offset: 0x1C */
    __IO uint32_t TABLE_CHG;        /*< Table change register,             Address offset: 0x20 */
    __IO uint32_t M_SCTRL;          /*< Memory special control,            Address offset: 0x24 */
} SBLOCK_TypeDef;

/*                                 
 * ==========================================================================
 * ---------------------- S Block registers bits ----------------------------
 * ==========================================================================
 */
/*< Memory special control register */
#define SB_MCTRL_EMA_0          ((uint32_t)( 0ul << 0))     /*< EMA 0 */
#define SB_MCTRL_EMA_1          ((uint32_t)( 1ul << 0))     /*< EMA 1 */
#define SB_MCTRL_EMA_2          ((uint32_t)( 2ul << 0))     /*< EMA 2 */
#define SB_MCTRL_EMA_3          ((uint32_t)( 3ul << 0))     /*< EMA 3 */
#define SB_MCTRL_EMA_4          ((uint32_t)( 4ul << 0))     /*< EMA 4 */
#define SB_MCTRL_EMA_5          ((uint32_t)( 5ul << 0))     /*< EMA 5 */
#define SB_MCTRL_EMA_6          ((uint32_t)( 6ul << 0))     /*< EMA 6 */
#define SB_MCTRL_EMA_7          ((uint32_t)( 7ul << 0))     /*< EMA 7 */ 

#define SB_MCTRL_EMAW_0         ((uint32_t)( 0ul << 3))     /*< EMAW 0 */
#define SB_MCTRL_EMAW_1         ((uint32_t)( 1ul << 3))     /*< EMAW 1 */
#define SB_MCTRL_EMAW_2         ((uint32_t)( 2ul << 3))     /*< EMAW 2 */
#define SB_MCTRL_EMAW_3         ((uint32_t)( 3ul << 3))     /*< EMAW 3 */

#define SB_MCTRL_FMS_0          ((uint32_t)( 0ul << 5))     /*< FMS 00 */
#define SB_MCTRL_FMS_1          ((uint32_t)( 1ul << 5))     /*< FMS 01 */
#define SB_MCTRL_FMS_2          ((uint32_t)( 2ul << 5))     /*< FMS 02 */
#define SB_MCTRL_FMS_3          ((uint32_t)( 3ul << 5))     /*< FMS 03 */
#define SB_MCTRL_FMS_4          ((uint32_t)( 4ul << 5))     /*< FMS 04 */
#define SB_MCTRL_FMS_5          ((uint32_t)( 5ul << 5))     /*< FMS 05 */
#define SB_MCTRL_FMS_6          ((uint32_t)( 6ul << 5))     /*< FMS 06 */
#define SB_MCTRL_FMS_7          ((uint32_t)( 7ul << 5))     /*< FMS 07 */ 
#define SB_MCTRL_FMS_8          ((uint32_t)( 8ul << 5))     /*< FMS 08 */
#define SB_MCTRL_FMS_9          ((uint32_t)( 9ul << 5))     /*< FMS 09 */
#define SB_MCTRL_FMS_10         ((uint32_t)(10ul << 5))     /*< FMS 10 */
#define SB_MCTRL_FMS_11         ((uint32_t)(11ul << 5))     /*< FMS 11 */
#define SB_MCTRL_FMS_12         ((uint32_t)(12ul << 5))     /*< FMS 12 */
#define SB_MCTRL_FMS_13         ((uint32_t)(13ul << 5))     /*< FMS 13 */
#define SB_MCTRL_FMS_14         ((uint32_t)(14ul << 5))     /*< FMS 14 */
#define SB_MCTRL_FMS_15         ((uint32_t)(15ul << 5))     /*< FMS 15 */ 

#define SB_MCTRL_WAWL_0         ((uint32_t)( 0ul << 9))     /*< WAWL 0 */
#define SB_MCTRL_WAWL_1         ((uint32_t)( 1ul << 9))     /*< WAWL 1 */ 

#define SB_MCTRL_WAWLM_0        ((uint32_t)(0ul << 10))     /*< WAWLM 0 */
#define SB_MCTRL_WAWLM_1        ((uint32_t)(1ul << 10))     /*< WAWLM 1 */ 
#define SB_MCTRL_WAWLM_2        ((uint32_t)(2ul << 10))     /*< WAWLM 2 */
#define SB_MCTRL_WAWLM_3        ((uint32_t)(3ul << 10))     /*< WAWLM 3 */

/*
 * ==========================================================================
 * ------------------------ L Block registers -------------------------------
 * ==========================================================================
 */
typedef struct
{
    __IO uint32_t TRM_00;               /*< Transformation register 0,         Address offset: 0x00 */
    __IO uint32_t TRM_01;               /*< Transformation register 1,         Address offset: 0x04 */
    __IO uint32_t TRM_02;               /*< Transformation register 2,         Address offset: 0x08 */
    __IO uint32_t TRM_03;               /*< Transformation register 3,         Address offset: 0x0C */
    __IO uint32_t TABLE_CHG_00;         /*< Table change 0 register,           Address offset: 0x10 */
    __IO uint32_t TABLE_CHG_01;         /*< Table change 1 register,           Address offset: 0x14 */
    __IO uint32_t TABLE_CHG_02;         /*< Table change 2 register,           Address offset: 0x18 */
    __IO uint32_t TABLE_CHG_03;         /*< Table change 3 register,           Address offset: 0x1C */
    __IO uint32_t SETUP;                /*< Setup register,                    Address offset: 0x20 */
    __IO uint32_t M_SCTRL;              /*< Memory special control,            Address offset: 0x24 */  
} LBLOCK_TypeDef;

/*
 * ==========================================================================
 * ------------------------ L Block registers bits --------------------------
 * ==========================================================================
 */
/*< Setup registers */
#define LB_SETUP_SET_RMASK      ((uint32_t)0x0000001FU)     /*< Set R Count mask */
#define LB_SETUP_CUR_RMASK      ((uint32_t)0x00001F00U)     /*< Current R Count mask */
#define LB_SETUP_SET_LDIR       ((uint32_t)0x00010000U)     /*< Set L Direction, 1 - reverse direction */
#define LB_SETUP_CUR_LDIR       ((uint32_t)0x00020000U)     /*< Current L Direction */
#define LB_SETUP_SET_BUSYEN     ((uint32_t)0x00040000U)     /*< Set Busy output enable */
#define LB_SETUP_CUR_BUSYEN     ((uint32_t)0x00080000U)     /*< Current Busy ontput enable state  */

/*< Memory special control register */
#define LB_MCTRL_EMA_0          ((uint32_t)( 0ul << 0))     /*< EMA 0 */
#define LB_MCTRL_EMA_1          ((uint32_t)( 1ul << 0))     /*< EMA 1 */
#define LB_MCTRL_EMA_2          ((uint32_t)( 2ul << 0))     /*< EMA 2 */
#define LB_MCTRL_EMA_3          ((uint32_t)( 3ul << 0))     /*< EMA 3 */
#define LB_MCTRL_EMA_4          ((uint32_t)( 4ul << 0))     /*< EMA 4 */
#define LB_MCTRL_EMA_5          ((uint32_t)( 5ul << 0))     /*< EMA 5 */
#define LB_MCTRL_EMA_6          ((uint32_t)( 6ul << 0))     /*< EMA 6 */
#define LB_MCTRL_EMA_7          ((uint32_t)( 7ul << 0))     /*< EMA 7 */ 

#define LB_MCTRL_EMAW_0         ((uint32_t)( 0ul << 3))     /*< EMAW 0 */
#define LB_MCTRL_EMAW_1         ((uint32_t)( 1ul << 3))     /*< EMAW 1 */
#define LB_MCTRL_EMAW_2         ((uint32_t)( 2ul << 3))     /*< EMAW 2 */
#define LB_MCTRL_EMAW_3         ((uint32_t)( 3ul << 3))     /*< EMAW 3 */

#define LB_MCTRL_FMS_0          ((uint32_t)( 0ul << 5))     /*< FMS 00 */
#define LB_MCTRL_FMS_1          ((uint32_t)( 1ul << 5))     /*< FMS 01 */
#define LB_MCTRL_FMS_2          ((uint32_t)( 2ul << 5))     /*< FMS 02 */
#define LB_MCTRL_FMS_3          ((uint32_t)( 3ul << 5))     /*< FMS 03 */
#define LB_MCTRL_FMS_4          ((uint32_t)( 4ul << 5))     /*< FMS 04 */
#define LB_MCTRL_FMS_5          ((uint32_t)( 5ul << 5))     /*< FMS 05 */
#define LB_MCTRL_FMS_6          ((uint32_t)( 6ul << 5))     /*< FMS 06 */
#define LB_MCTRL_FMS_7          ((uint32_t)( 7ul << 5))     /*< FMS 07 */ 
#define LB_MCTRL_FMS_8          ((uint32_t)( 8ul << 5))     /*< FMS 08 */
#define LB_MCTRL_FMS_9          ((uint32_t)( 9ul << 5))     /*< FMS 09 */
#define LB_MCTRL_FMS_10         ((uint32_t)(10ul << 5))     /*< FMS 10 */
#define LB_MCTRL_FMS_11         ((uint32_t)(11ul << 5))     /*< FMS 11 */
#define LB_MCTRL_FMS_12         ((uint32_t)(12ul << 5))     /*< FMS 12 */
#define LB_MCTRL_FMS_13         ((uint32_t)(13ul << 5))     /*< FMS 13 */
#define LB_MCTRL_FMS_14         ((uint32_t)(14ul << 5))     /*< FMS 14 */
#define LB_MCTRL_FMS_15         ((uint32_t)(15ul << 5))     /*< FMS 15 */ 

#define LB_MCTRL_WAWL_0         ((uint32_t)( 0ul << 9))     /*< WAWL 0 */
#define LB_MCTRL_WAWL_1         ((uint32_t)( 1ul << 9))     /*< WAWL 1 */ 

#define LB_MCTRL_WAWLM_0        ((uint32_t)(0ul << 10))     /*< WAWLM 0 */
#define LB_MCTRL_WAWLM_1        ((uint32_t)(1ul << 10))     /*< WAWLM 1 */ 
#define LB_MCTRL_WAWLM_2        ((uint32_t)(2ul << 10))     /*< WAWLM 2 */
#define LB_MCTRL_WAWLM_3        ((uint32_t)(3ul << 10))     /*< WAWLM 3 */

/*
 * ==========================================================================
 * -------------------------- DES registers ---------------------------------
 * ==========================================================================
 */
typedef struct
{
    __IO uint32_t TRM;              /*< Transformation register,           Address offset: 0x00 */
    __IO uint32_t KEY_L;            /*< Key low part register,             Address offset: 0x04 */
    __IO uint32_t KEY_H;            /*< Key high part register,            Address offset: 0x08 */
    __IO uint32_t E_MUX_SET;        /*< E Transformation setup register,   Address offset: 0x0C */
    __IO uint32_t E_MUX_VAL;        /*< E Transformation value register,   Address offset: 0x10 */
    __IO uint32_t P_MUX_SET;        /*< P Transformation setup register,   Address offset: 0x14 */
    __IO uint32_t P_MUX_VAL;        /*< P Transformation value register,   Address offset: 0x18 */
    __IO uint32_t STABLE;           /*< S table register,                  Address offset: 0x1C */
    __IO uint32_t M_SCTRL;          /*< Memory special control,            Address offset: 0x20 */  
} DES_TypeDef;

/*                                 
 * ==========================================================================
 * ------------------------ DES registers bits ------------------------------
 * ==========================================================================
 */
/*< Memory special control register */
#define DES_MCTRL_EMA_0          ((uint32_t)( 0ul << 0))        /*< EMA 0 */
#define DES_MCTRL_EMA_1          ((uint32_t)( 1ul << 0))        /*< EMA 1 */
#define DES_MCTRL_EMA_2          ((uint32_t)( 2ul << 0))        /*< EMA 2 */
#define DES_MCTRL_EMA_3          ((uint32_t)( 3ul << 0))        /*< EMA 3 */
#define DES_MCTRL_EMA_4          ((uint32_t)( 4ul << 0))        /*< EMA 4 */
#define DES_MCTRL_EMA_5          ((uint32_t)( 5ul << 0))        /*< EMA 5 */
#define DES_MCTRL_EMA_6          ((uint32_t)( 6ul << 0))        /*< EMA 6 */
#define DES_MCTRL_EMA_7          ((uint32_t)( 7ul << 0))        /*< EMA 7 */ 

#define DES_MCTRL_EMAW_0         ((uint32_t)( 0ul << 3))        /*< EMAW 0 */
#define DES_MCTRL_EMAW_1         ((uint32_t)( 1ul << 3))        /*< EMAW 1 */
#define DES_MCTRL_EMAW_2         ((uint32_t)( 2ul << 3))        /*< EMAW 2 */
#define DES_MCTRL_EMAW_3         ((uint32_t)( 3ul << 3))        /*< EMAW 3 */

#define DES_MCTRL_FMS_0          ((uint32_t)( 0ul << 5))        /*< FMS 00 */
#define DES_MCTRL_FMS_1          ((uint32_t)( 1ul << 5))        /*< FMS 01 */
#define DES_MCTRL_FMS_2          ((uint32_t)( 2ul << 5))        /*< FMS 02 */
#define DES_MCTRL_FMS_3          ((uint32_t)( 3ul << 5))        /*< FMS 03 */
#define DES_MCTRL_FMS_4          ((uint32_t)( 4ul << 5))        /*< FMS 04 */
#define DES_MCTRL_FMS_5          ((uint32_t)( 5ul << 5))        /*< FMS 05 */
#define DES_MCTRL_FMS_6          ((uint32_t)( 6ul << 5))        /*< FMS 06 */
#define DES_MCTRL_FMS_7          ((uint32_t)( 7ul << 5))        /*< FMS 07 */ 
#define DES_MCTRL_FMS_8          ((uint32_t)( 8ul << 5))        /*< FMS 08 */
#define DES_MCTRL_FMS_9          ((uint32_t)( 9ul << 5))        /*< FMS 09 */
#define DES_MCTRL_FMS_10         ((uint32_t)(10ul << 5))        /*< FMS 10 */
#define DES_MCTRL_FMS_11         ((uint32_t)(11ul << 5))        /*< FMS 11 */
#define DES_MCTRL_FMS_12         ((uint32_t)(12ul << 5))        /*< FMS 12 */
#define DES_MCTRL_FMS_13         ((uint32_t)(13ul << 5))        /*< FMS 13 */
#define DES_MCTRL_FMS_14         ((uint32_t)(14ul << 5))        /*< FMS 14 */
#define DES_MCTRL_FMS_15         ((uint32_t)(15ul << 5))        /*< FMS 15 */ 

#define DES_MCTRL_WAWL_0         ((uint32_t)( 0ul << 9))        /*< WAWL 0 */
#define DES_MCTRL_WAWL_1         ((uint32_t)( 1ul << 9))        /*< WAWL 1 */ 

#define DES_MCTRL_WAWLM_0        ((uint32_t)(0ul << 10))        /*< WAWLM 0 */
#define DES_MCTRL_WAWLM_1        ((uint32_t)(1ul << 10))        /*< WAWLM 1 */ 
#define DES_MCTRL_WAWLM_2        ((uint32_t)(2ul << 10))        /*< WAWLM 2 */
#define DES_MCTRL_WAWLM_3        ((uint32_t)(3ul << 10))        /*< WAWLM 3 */

/*
 * ==========================================================================
 * -------------------------- ALU registers ---------------------------------
 * ==========================================================================
 */
typedef struct
{
    __IO uint32_t CMDC_CTRL;        /*< Command counter control register       Address offset: 0x00 */
    __IO uint32_t IF;               /*< interrupt flag register                Address offset: 0x04 */
    __IO uint32_t IE;               /*< interrupt enable register              Address offset: 0x08 */
    __IO uint32_t ALU_SCTRL;        /*< ALU status and control register        Address offset: 0x0C */
    __IO uint32_t BASE_SHIFT;       /*< ALU base shift register                Address offset: 0x10 */  
    __IO uint32_t M_SCTRL;          /*< Memory special control,                Address offset: 0x14 */
} ALU_TypeDef;

/*                                 
 * ==========================================================================
 * ------------------------ ALU registers bits ------------------------------
 * ==========================================================================
 */
/*< Memory special control register */
#define ALU_MCTRL_EMA_0         ((uint32_t)( 0ul << 0))     /*< EMA 0 */
#define ALU_MCTRL_EMA_1         ((uint32_t)( 1ul << 0))     /*< EMA 1 */
#define ALU_MCTRL_EMA_2         ((uint32_t)( 2ul << 0))     /*< EMA 2 */
#define ALU_MCTRL_EMA_3         ((uint32_t)( 3ul << 0))     /*< EMA 3 */
#define ALU_MCTRL_EMA_4         ((uint32_t)( 4ul << 0))     /*< EMA 4 */
#define ALU_MCTRL_EMA_5         ((uint32_t)( 5ul << 0))     /*< EMA 5 */
#define ALU_MCTRL_EMA_6         ((uint32_t)( 6ul << 0))     /*< EMA 6 */
#define ALU_MCTRL_EMA_7         ((uint32_t)( 7ul << 0))     /*< EMA 7 */ 

#define ALU_MCTRL_EMAW_0        ((uint32_t)( 0ul << 3))     /*< EMAW 0 */
#define ALU_MCTRL_EMAW_1        ((uint32_t)( 1ul << 3))     /*< EMAW 1 */
#define ALU_MCTRL_EMAW_2        ((uint32_t)( 2ul << 3))     /*< EMAW 2 */
#define ALU_MCTRL_EMAW_3        ((uint32_t)( 3ul << 3))     /*< EMAW 3 */

#define ALU_MCTRL_FMS_0         ((uint32_t)( 0ul << 5))     /*< FMS 00 */
#define ALU_MCTRL_FMS_1         ((uint32_t)( 1ul << 5))     /*< FMS 01 */
#define ALU_MCTRL_FMS_2         ((uint32_t)( 2ul << 5))     /*< FMS 02 */
#define ALU_MCTRL_FMS_3         ((uint32_t)( 3ul << 5))     /*< FMS 03 */
#define ALU_MCTRL_FMS_4         ((uint32_t)( 4ul << 5))     /*< FMS 04 */
#define ALU_MCTRL_FMS_5         ((uint32_t)( 5ul << 5))     /*< FMS 05 */
#define ALU_MCTRL_FMS_6         ((uint32_t)( 6ul << 5))     /*< FMS 06 */
#define ALU_MCTRL_FMS_7         ((uint32_t)( 7ul << 5))     /*< FMS 07 */ 
#define ALU_MCTRL_FMS_8         ((uint32_t)( 8ul << 5))     /*< FMS 08 */
#define ALU_MCTRL_FMS_9         ((uint32_t)( 9ul << 5))     /*< FMS 09 */
#define ALU_MCTRL_FMS_10        ((uint32_t)(10ul << 5))     /*< FMS 10 */
#define ALU_MCTRL_FMS_11        ((uint32_t)(11ul << 5))     /*< FMS 11 */
#define ALU_MCTRL_FMS_12        ((uint32_t)(12ul << 5))     /*< FMS 12 */
#define ALU_MCTRL_FMS_13        ((uint32_t)(13ul << 5))     /*< FMS 13 */
#define ALU_MCTRL_FMS_14        ((uint32_t)(14ul << 5))     /*< FMS 14 */
#define ALU_MCTRL_FMS_15        ((uint32_t)(15ul << 5))     /*< FMS 15 */ 

#define ALU_MCTRL_WAWL_0        ((uint32_t)( 0ul << 9))     /*< WAWL 0 */
#define ALU_MCTRL_WAWL_1        ((uint32_t)( 1ul << 9))     /*< WAWL 1 */ 

#define ALU_MCTRL_WAWLM_0       ((uint32_t)(0ul << 10))     /*< WAWLM 0 */
#define ALU_MCTRL_WAWLM_1       ((uint32_t)(1ul << 10))     /*< WAWLM 1 */ 
#define ALU_MCTRL_WAWLM_2       ((uint32_t)(2ul << 10))     /*< WAWLM 2 */
#define ALU_MCTRL_WAWLM_3       ((uint32_t)(3ul << 10))     /*< WAWLM 3 */

/*< Command counter control register */
#define CMDC_START_ADDR_MASK    ((uint32_t)0x000003FF)                      /*< Command series starting address */
#define CMDC_START_ADDR_GET(r)  ((uint32_t)(r & CMDC_START_ADDR_MASK))      /*< Get from register               */
#define CMDC_START_ADDR_SET(v)  (CMDC_START_ADDR_GET(v))                    /*< Set to register                 */

#define CMDC_SEQ_NUM_MASK       ((uint32_t)0x0000FC00)                      /*< Command series number           */
#define CMDC_SEQ_NUM_GET(r)     ((uint32_t)((r & CMDC_SEQ_NUM_MASK) >> 10)) /*< Get from register               */
#define CMDC_SEQ_NUM_SET(v)     ((uint32_t)((v << 10) & CMDC_SEQ_NUM_MASK)) /*< Set to register                 */

#define CMDC_CMD_NUM_MASK       ((uint32_t)0x07FF0000)                      /*< Native commands count in series */
#define CMDC_CMD_NUM_GET(r)     ((uint32_t)((r & CMDC_CMD_NUM_MASK) >> 16)) /*< Get from register               */
#define CMDC_CMD_NUM_SET(v)     ((uint32_t)((v << 16) & CMDC_CMD_NUM_MASK)) /*< Set to register                 */

#define CMDC_BANK_SEL           ((uint32_t)0x80000000)                      /*< Commands bank bit               */
#define CMDC_BANK_SEL_0         ((uint32_t)0x00000000)                      /*< Select commands bank 0          */
#define CMDC_BANK_SEL_1         ((uint32_t)0x80000000)                      /*< Select commands bank 1          */

/*< interrupt flag register */
#define IF_END                  ((uint32_t)0x00000001)      /*< End of command series flag */
#define IF_WCODE                ((uint32_t)0x00000002)      /*< Error. Wrong command flag */
#define IF_WOPER                ((uint32_t)0x00000004)      /*< Error. Wrong parameter flag */

/*< interrupt enable register */
#define IE_END                  ((uint32_t)0x00000001)      /*< Enable/disable end of command series interrupt */
#define IE_WCODE                ((uint32_t)0x00000002)      /*< Enable/disable wrong command interrupt  */
#define IE_WOPER                ((uint32_t)0x00000004)      /*< Enable/disable wrong parameter interrupt */

/*< ALU status and control register */
#define CARRY                   ((uint32_t)0x00000001)      /*< Carry bit */
#define SHIFT                   ((uint32_t)0x00000002)      /*< Shift bit */
#define SET_CARRY               ((uint32_t)0x00000004)      /*< Set carry bit */
#define SET_SHIFT               ((uint32_t)0x00000008)      /*< Set shift bit */
#define CLR_CARRY               ((uint32_t)0x00000010)      /*< Clear carry bit */
#define CLR_SHIFT               ((uint32_t)0x00000020)      /*< Clear shift bit */

/*< ALU opcodes */
#define NOP_CMD                 ((uint32_t)  0)
#define MOV_CMD                 ((uint32_t)  1)
#define C_MOV_CMD               ((uint32_t)  2)
#define SHL_CMD                 ((uint32_t) 10)
#define C_SHL_CMD               ((uint32_t) 11)
#define S_SHL_CMD               ((uint32_t) 12)
#define O_SHL_CMD               ((uint32_t) 13)
#define SHR_CMD                 ((uint32_t) 15)
#define C_SHR_CMD               ((uint32_t) 16)
#define S_SHR_CMD               ((uint32_t) 17)
#define O_SHR_CMD               ((uint32_t) 18)
#define ADD_CMD                 ((uint32_t) 20)
#define C_ADD_CMD               ((uint32_t) 21)
#define O_ADD_CMD               ((uint32_t) 22)
#define MUL_CMD                 ((uint32_t) 25)
#define PUSH_CMD                ((uint32_t) 30)
#define POP_CMD                 ((uint32_t) 31)
#define B_SHFT_CMD              ((uint32_t)255)

/*
 * ==========================================================================
 * -------------------------- OTP registers ---------------------------------
 * ==========================================================================
 */
typedef struct
{
    __IO uint32_t STAT_CTRL_REG;            /*< Status and control register         Address offset: 0x00 */
    __IO uint32_t OTP_STAT_REG;             /*< Blocks status register              Address offset: 0x04 */
    __IO uint32_t DELAY_0_REG;              /*< Delay 0 register                    Address offset: 0x08 */
    __IO uint32_t DELAY_1_REG;              /*< Delay 1 register                    Address offset: 0x0C */
    __IO uint32_t RW_CMD_REG;               /*< Read/write command register         Address offset: 0x10 */
    __IO uint32_t READ_DATA_REG;            /*< Read data register                  Address offset: 0x14 */
    __IO uint32_t WRITE_PROTECT_REG;        /*< Write protection register           Address offset: 0x18 */
    __IO uint32_t READ_PROTECT_REG;         /*< Delay 0 register                    Address offset: 0x1C */
    __IO uint32_t DIR_CTRL_REG;             /*< Direct control register             Address offset: 0x20 */
    __IO uint32_t DIR_DATA_REG;             /*< Direct control data register        Address offset: 0x24 */
    __IO uint32_t TEST_OPT_REG;             /*< Test options register               Address offset: 0x28 */
} OTP_TypeDef;

/*                                 
 * ==========================================================================
 * ------------------------ OTP registers bits ------------------------------
 * ==========================================================================
 */
/*< Status and control register */
#define OTP_FSM_STATE_MASK          ((uint32_t)0x00000007)      /*< Fsm state mask                       */
#define OTP_FSM_IDLE                ((uint32_t)0x00000000)      /*< Fsm idle state                       */
#define OTP_FSM_INIT                ((uint32_t)0x00000001)      /*< Fsm init state                       */
#define OTP_FSM_READY               ((uint32_t)0x00000002)      /*< Fsm ready state                      */
#define OTP_FSM_REG_READ_PENDING    ((uint32_t)0x00000003)      /*< Fsm read via registers pending state */
#define OTP_FSM_BUS_READ_PENDING    ((uint32_t)0x00000004)      /*< Fsm read via bus pending state       */
#define OTP_FSM_WRITE_PENDING       ((uint32_t)0x00000005)      /*< Fsm write pending state              */
#define OTP_FSM_DIR_CTRL            ((uint32_t)0x00000006)      /*< Fsm direct control state             */

#define OTP_BUSY                    ((uint32_t)0x00000008)      /*< OTP is busy                                     */
#define OTP_LOCK                    ((uint32_t)0x00000010)      /*< Entire OTP write lock                           */
#define OTP_PLOCK                   ((uint32_t)0x00000020)      /*< Partly OTP write lock                           */
#define OTP_RWERROR                 ((uint32_t)0x00000040)      /*< Read/write error                                */
#define OTP_TEST_CRC                ((uint32_t)0x00000080)      /*< Special test data integrity signal              */
#define OTP_READ_PCRC               ((uint32_t)0x00000100)      /*< Read protection register data integrity signal  */
#define OTP_WRITE_PCRC              ((uint32_t)0x00000200)      /*< Write protection register data integrity signal */
#define OTP_DIR_EN                  ((uint32_t)0x00000400)      /*< Direct control request bit                      */
#define OTP_DIR_ACK                 ((uint32_t)0x00000800)      /*< Direct control acknowledge bit                  */
#define OTP_DIR_ERR                 ((uint32_t)0x00001000)      /*< Direct control error bit                        */

/*< Blocks status register */
#define OTP_0_FSM_STATE_MASK                ((uint32_t)0x0000001F)      /*< OTP0 block fsm state mask                                 */
#define OTP_0_FSM_IDLE                      ((uint32_t)0x00000000)      /*< OTP0 block fsm idle state                                 */
#define OTP_0_FSM_ENERGY_SAVE_PENDING       ((uint32_t)0x00000001)      /*< OTP0 block fsm energy save pending state                  */
#define OTP_0_FSM_RESET_PENDING             ((uint32_t)0x00000002)      /*< OTP0 block fsm reset pending state                        */
#define OTP_0_FSM_RESET                     ((uint32_t)0x00000003)      /*< OTP0 block fsm reset state                                */
#define OTP_0_FSM_RESET_ENDING              ((uint32_t)0x00000004)      /*< OTP0 block fsm reset finalization state                   */
#define OTP_0_FSM_ACTIVE                    ((uint32_t)0x00000005)      /*< OTP0 block fsm active state. Read/write pending           */
#define OTP_0_FSM_PAUSE                     ((uint32_t)0x00000006)      /*< OTP0 block fsm pause state. After write                   */
#define OTP_0_FSM_READ_PENDING              ((uint32_t)0x00000007)      /*< OTP0 block fsm read pending state                         */
#define OTP_0_FSM_READ                      ((uint32_t)0x00000008)      /*< OTP0 block fsm read state                                 */
#define OTP_0_FSM_READ_ENDING               ((uint32_t)0x00000009)      /*< OTP0 block fsm read finalization state                    */
#define OTP_0_FSM_WRITE_PENDING             ((uint32_t)0x0000000A)      /*< OTP0 block fsm write pending state                        */
#define OTP_0_FSM_WRITE_DATA_SAVE           ((uint32_t)0x0000000B)      /*< OTP0 block fsm saving data state                          */
#define OTP_0_FSM_WRITE_DATA_SAVE_ENDING    ((uint32_t)0x0000000C)      /*< OTP0 block fsm finalization of saving data state          */
#define OTP_0_FSM_PROGR_PENDING             ((uint32_t)0x0000000D)      /*< OTP0 block fsm programming pending state                  */
#define OTP_0_FSM_HIGH_VOLTAGE_PENDING      ((uint32_t)0x0000000E)      /*< OTP0 block fsm voltage increase circuit preparing state   */
#define OTP_0_FSM_PROGR                     ((uint32_t)0x0000000F)      /*< OTP0 block fsm programming state                          */
#define OTP_0_FSM_HIGH_VOLTAGE_ENDING       ((uint32_t)0x00000010)      /*< OTP0 block fsm voltage increase circuit turning off state */
#define OTP_0_FSM_PROGR_ENDING              ((uint32_t)0x00000011)      /*< OTP0 block fsm programming finalization state             */
#define OTP_0_FSM_WRITE_ENDING              ((uint32_t)0x00000012)      /*< OTP0 block fsm write finalization state                   */
#define OTP_0_FSM_DIR_CTRL_PENDING          ((uint32_t)0x00000013)      /*< OTP0 block fsm direct control pending state               */
#define OTP_0_FSM_DIR_CTRL                  ((uint32_t)0x00000014)      /*< OTP0 block fsm direct control state                       */
                                                                        /*< 0x00000020 - reserved                                     */
#define OTP_0_LOCK                          ((uint32_t)0x00000040)      /*< OTP0 block is locked for writing                          */
#define OTP_0_BUSY                          ((uint32_t)0x00000080)      /*< OTP0 block is busy                                        */

#define OTP_1_FSM_STATE_MASK                ((uint32_t)0x00001F00)      /*< OTP1 block fsm state mask                                 */
#define OTP_1_FSM_IDLE                      ((uint32_t)0x00000000)      /*< OTP1 block fsm idle state                                 */
#define OTP_1_FSM_ENERGY_SAVE_PENDING       ((uint32_t)0x00000100)      /*< OTP1 block fsm energy save pending state                  */
#define OTP_1_FSM_RESET_PENDING             ((uint32_t)0x00000200)      /*< OTP1 block fsm reset pending state                        */
#define OTP_1_FSM_RESET                     ((uint32_t)0x00000300)      /*< OTP1 block fsm reset state                                */
#define OTP_1_FSM_RESET_ENDING              ((uint32_t)0x00000400)      /*< OTP1 block fsm reset finalization state                   */
#define OTP_1_FSM_ACTIVE                    ((uint32_t)0x00000500)      /*< OTP1 block fsm active state. Read/write pending           */
#define OTP_1_FSM_PAUSE                     ((uint32_t)0x00000600)      /*< OTP1 block fsm pause state. After write                   */
#define OTP_1_FSM_READ_PENDING              ((uint32_t)0x00000700)      /*< OTP1 block fsm read pending state                         */
#define OTP_1_FSM_READ                      ((uint32_t)0x00000800)      /*< OTP1 block fsm read state                                 */
#define OTP_1_FSM_READ_ENDING               ((uint32_t)0x00000900)      /*< OTP1 block fsm read finalization state                    */
#define OTP_1_FSM_WRITE_PENDING             ((uint32_t)0x00000A00)      /*< OTP1 block fsm write pending state                        */
#define OTP_1_FSM_WRITE_DATA_SAVE           ((uint32_t)0x00000B00)      /*< OTP1 block fsm saving data state                          */
#define OTP_1_FSM_WRITE_DATA_SAVE_ENDING    ((uint32_t)0x00000C00)      /*< OTP1 block fsm finalization of saving data state          */
#define OTP_1_FSM_PROGR_PENDING             ((uint32_t)0x00000D00)      /*< OTP1 block fsm programming pending state                  */
#define OTP_1_FSM_HIGH_VOLTAGE_PENDING      ((uint32_t)0x00000E00)      /*< OTP1 block fsm voltage increase circuit preparing state   */
#define OTP_1_FSM_PROGR                     ((uint32_t)0x00000F00)      /*< OTP1 block fsm programming state                          */
#define OTP_1_FSM_HIGH_VOLTAGE_ENDING       ((uint32_t)0x00001000)      /*< OTP1 block fsm voltage increase circuit turning off state */
#define OTP_1_FSM_PROGR_ENDING              ((uint32_t)0x00001100)      /*< OTP1 block fsm programming finalization state             */
#define OTP_1_FSM_WRITE_ENDING              ((uint32_t)0x00001200)      /*< OTP1 block fsm write finalization state                   */
#define OTP_1_FSM_DIR_CTRL_PENDING          ((uint32_t)0x00001300)      /*< OTP1 block fsm direct control pending state               */
#define OTP_1_FSM_DIR_CTRL                  ((uint32_t)0x00001400)      /*< OTP1 block fsm direct control state                       */
                                                                        /*< 0x00002000 - reserved                                     */
#define OTP_1_LOCK                          ((uint32_t)0x00004000)      /*< OTP1 block is locked for writing                          */
#define OTP_1_BUSY                          ((uint32_t)0x00008000)      /*< OTP1 block is busy                                        */

#define OTP_2_FSM_STATE_MASK                ((uint32_t)0x001F0000)      /*< OTP2 block fsm state mask                                 */
#define OTP_2_FSM_IDLE                      ((uint32_t)0x00000000)      /*< OTP2 block fsm idle state                                 */
#define OTP_2_FSM_ENERGY_SAVE_PENDING       ((uint32_t)0x00010000)      /*< OTP2 block fsm energy save pending state                  */
#define OTP_2_FSM_RESET_PENDING             ((uint32_t)0x00020000)      /*< OTP2 block fsm reset pending state                        */
#define OTP_2_FSM_RESET                     ((uint32_t)0x00030000)      /*< OTP2 block fsm reset state                                */
#define OTP_2_FSM_RESET_ENDING              ((uint32_t)0x00040000)      /*< OTP2 block fsm reset finalization state                   */
#define OTP_2_FSM_ACTIVE                    ((uint32_t)0x00050000)      /*< OTP2 block fsm active state. Read/write pending           */
#define OTP_2_FSM_PAUSE                     ((uint32_t)0x00060000)      /*< OTP2 block fsm pause state. After write                   */
#define OTP_2_FSM_READ_PENDING              ((uint32_t)0x00070000)      /*< OTP2 block fsm read pending state                         */
#define OTP_2_FSM_READ                      ((uint32_t)0x00080000)      /*< OTP2 block fsm read state                                 */
#define OTP_2_FSM_READ_ENDING               ((uint32_t)0x00090000)      /*< OTP2 block fsm read finalization state                    */
#define OTP_2_FSM_WRITE_PENDING             ((uint32_t)0x000A0000)      /*< OTP2 block fsm write pending state                        */
#define OTP_2_FSM_WRITE_DATA_SAVE           ((uint32_t)0x000B0000)      /*< OTP2 block fsm saving data state                          */
#define OTP_2_FSM_WRITE_DATA_SAVE_ENDING    ((uint32_t)0x000C0000)      /*< OTP2 block fsm finalization of saving data state          */
#define OTP_2_FSM_PROGR_PENDING             ((uint32_t)0x000D0000)      /*< OTP2 block fsm programming pending state                  */
#define OTP_2_FSM_HIGH_VOLTAGE_PENDING      ((uint32_t)0x000E0000)      /*< OTP2 block fsm voltage increase circuit preparing state   */
#define OTP_2_FSM_PROGR                     ((uint32_t)0x000F0000)      /*< OTP2 block fsm programming state                          */
#define OTP_2_FSM_HIGH_VOLTAGE_ENDING       ((uint32_t)0x00100000)      /*< OTP2 block fsm voltage increase circuit turning off state */
#define OTP_2_FSM_PROGR_ENDING              ((uint32_t)0x00110000)      /*< OTP2 block fsm programming finalization state             */
#define OTP_2_FSM_WRITE_ENDING              ((uint32_t)0x00120000)      /*< OTP2 block fsm write finalization state                   */
#define OTP_2_FSM_DIR_CTRL_PENDING          ((uint32_t)0x00130000)      /*< OTP2 block fsm direct control pending state               */
#define OTP_2_FSM_DIR_CTRL                  ((uint32_t)0x00140000)      /*< OTP2 block fsm direct control state                       */
                                                                        /*< 0x00200000 - reserved                                     */
#define OTP_2_LOCK                          ((uint32_t)0x00400000)      /*< OTP2 block is locked for writing                          */
#define OTP_2_BUSY                          ((uint32_t)0x00800000)      /*< OTP2 block is busy                                        */

#define OTP_3_FSM_STATE_MASK                ((uint32_t)0x1F000000)      /*< OTP3 block fsm state mask                                 */
#define OTP_3_FSM_IDLE                      ((uint32_t)0x00000000)      /*< OTP3 block fsm idle state                                 */
#define OTP_3_FSM_ENERGY_SAVE_PENDING       ((uint32_t)0x01000000)      /*< OTP3 block fsm energy save pending state                  */
#define OTP_3_FSM_RESET_PENDING             ((uint32_t)0x02000000)      /*< OTP3 block fsm reset pending state                        */
#define OTP_3_FSM_RESET                     ((uint32_t)0x03000000)      /*< OTP3 block fsm reset state                                */
#define OTP_3_FSM_RESET_ENDING              ((uint32_t)0x04000000)      /*< OTP3 block fsm reset finalization state                   */
#define OTP_3_FSM_ACTIVE                    ((uint32_t)0x05000000)      /*< OTP3 block fsm active state. Read/write pending           */
#define OTP_3_FSM_PAUSE                     ((uint32_t)0x06000000)      /*< OTP3 block fsm pause state. After write                   */
#define OTP_3_FSM_READ_PENDING              ((uint32_t)0x07000000)      /*< OTP3 block fsm read pending state                         */
#define OTP_3_FSM_READ                      ((uint32_t)0x08000000)      /*< OTP3 block fsm read state                                 */
#define OTP_3_FSM_READ_ENDING               ((uint32_t)0x09000000)      /*< OTP3 block fsm read finalization state                    */
#define OTP_3_FSM_WRITE_PENDING             ((uint32_t)0x0A000000)      /*< OTP3 block fsm write pending state                        */
#define OTP_3_FSM_WRITE_DATA_SAVE           ((uint32_t)0x0B000000)      /*< OTP3 block fsm saving data state                          */
#define OTP_3_FSM_WRITE_DATA_SAVE_ENDING    ((uint32_t)0x0C000000)      /*< OTP3 block fsm finalization of saving data state          */
#define OTP_3_FSM_PROGR_PENDING             ((uint32_t)0x0D000000)      /*< OTP3 block fsm programming pending state                  */
#define OTP_3_FSM_HIGH_VOLTAGE_PENDING      ((uint32_t)0x0E000000)      /*< OTP3 block fsm voltage increase circuit preparing state   */
#define OTP_3_FSM_PROGR                     ((uint32_t)0x0F000000)      /*< OTP3 block fsm programming state                          */
#define OTP_3_FSM_HIGH_VOLTAGE_ENDING       ((uint32_t)0x10000000)      /*< OTP3 block fsm voltage increase circuit turning off state */
#define OTP_3_FSM_PROGR_ENDING              ((uint32_t)0x11000000)      /*< OTP3 block fsm programming finalization state             */
#define OTP_3_FSM_WRITE_ENDING              ((uint32_t)0x12000000)      /*< OTP3 block fsm write finalization state                   */
#define OTP_3_FSM_DIR_CTRL_PENDING          ((uint32_t)0x13000000)      /*< OTP3 block fsm direct control pending state               */
#define OTP_3_FSM_DIR_CTRL                  ((uint32_t)0x14000000)      /*< OTP3 block fsm direct control state                       */
                                                                        /*< 0x20000000 - reserved                                     */
#define OTP_3_LOCK                          ((uint32_t)0x40000000)      /*< OTP3 block is locked for writing                          */
#define OTP_3_BUSY                          ((uint32_t)0x80000000)      /*< OTP3 block is busy                                        */

/*< Read/write command register */
#define OTP_CMD_READ                        ((uint32_t)0x10000000)      /*< OTP read command                 */
#define OTP_CMD_WRITE                       ((uint32_t)0x20000000)      /*< OTP write command                */
#define OTP_CMD_RW_ERROR                    ((uint32_t)0x80000000)      /*< OTP last command execution error */

/*< Direct control register */
#define OTP_DIR_CTRL_WEB                    ((uint32_t)0x00000001)      /*< Controls the corresponding block signal */
#define OTP_DIR_CTRL_RSTB                   ((uint32_t)0x00000002)      /*< Controls the corresponding block signal */
#define OTP_DIR_CTRL_READEN                 ((uint32_t)0x00000004)      /*< Controls the corresponding block signal */
#define OTP_DIR_CTRL_PGMEN                  ((uint32_t)0x00000008)      /*< Controls the corresponding block signal */
#define OTP_DIR_CTRL_DLE                    ((uint32_t)0x00000010)      /*< Controls the corresponding block signal */
#define OTP_DIR_CTRL_DIN_0                  ((uint32_t)0x00000020)      /*< Controls the corresponding block signal */
#define OTP_DIR_CTRL_DIN_1                  ((uint32_t)0x00000040)      /*< Controls the corresponding block signal */
#define OTP_DIR_CTRL_DIN_2                  ((uint32_t)0x00000080)      /*< Controls the corresponding block signal */
#define OTP_DIR_CTRL_DIN_3                  ((uint32_t)0x00000100)      /*< Controls the corresponding block signal */
#define OTP_DIR_CTRL_CPUMPEN                ((uint32_t)0x00000200)      /*< Controls the corresponding block signal */
#define OTP_DIR_CTRL_CLE                    ((uint32_t)0x00000400)      /*< Controls the corresponding block signal */
#define OTP_DIR_CTRL_CEB                    ((uint32_t)0x00000800)      /*< Controls the corresponding block signal */
#define OTP_DIR_CTRL_A_MASK                 ((uint32_t)0x1FFFF000)      /*< Controls the corresponding block signal */

/*
 * ==========================================================================
 * ------------------------------- GATE -------------------------------------
 * ==========================================================================
 */
#define GATE_REG_0      ((uint8_t)0)
#define GATE_REG_1      ((uint8_t)1)
#define GATE_REG_2      ((uint8_t)2)
#define GATE_REG_3      ((uint8_t)3)
#define GATE_REG_4      ((uint8_t)4)
#define GATE_REG_5      ((uint8_t)5)
#define GATE_REG_6      ((uint8_t)6)
#define GATE_REG_7      ((uint8_t)7)
#define GATE_REG_8      ((uint8_t)8)
#define GATE_REG_9      ((uint8_t)9)
#define GATE_REG_10     ((uint8_t)10)
#define GATE_REG_11     ((uint8_t)11)
#define GATE_REG_12     ((uint8_t)12)
#define GATE_REG_13     ((uint8_t)13)
#define GATE_REG_14     ((uint8_t)14)
#define GATE_REG_15     ((uint8_t)15)
#define GATE_REG_CNT    ((uint8_t)16)

typedef struct
{
    __IO uint32_t SF_FIFO_OP_TO_SF;         /*< FIFO opened core -> safe core data register    Address offset: 0x00 */
    __IO uint32_t SF_FIFO_SF_TO_OP;         /*< FIFO safe core -> opened core data register    Address offset: 0x04 */
    __IO uint32_t SF_FIFO_LEVELS;           /*< FIFO control levels register                   Address offset: 0x08 */
    __IO uint32_t SF_FIFO_INT_MASK;         /*< FIFO interruptions mask register               Address offset: 0x0C */
    __IO uint32_t SF_FIFO_INT_SOURCE;       /*< FIFO interruption source register              Address offset: 0x10 */
    __IO uint32_t SF_REG[GATE_REG_CNT];     /*< In/out data registers                          Address offset: 0x14 */
    __IO uint32_t SF_REGS_BUSY;             /*< Input data registers busyness indicator        Address offset: 0x54 */
    __IO uint32_t SF_REGS_INT_MASK;         /*< Data registers interruptions mask register     Address offset: 0x58 */
    __IO uint32_t SF_REGS_INT_SOURCE;       /*< Data registers interruption source register    Address offset: 0x5C */
} GATE_TypeDef;

/*                                 
 * ==========================================================================
 * ------------------------ GATE registers bits -----------------------------
 * ==========================================================================
 */
/*< FIFO interruption source register */
#define FLAG_SF_FIFO_IN_EMPTY               ((uint32_t)0x00000001)      /*< SF2OP FIFO is empty                            */
#define FLAG_SF_FIFO_IN_LOWER               ((uint32_t)0x00000002)      /*< SF2OP FIFO is filled below the specified level */
#define FLAG_SF_FIFO_IN_NOT_FULL            ((uint32_t)0x00000004)      /*< SF2OP FIFO is not full                         */
#define FLAG_SF_FIFO_IN_WR_TO_FULL          ((uint32_t)0x00000008)      /*< Write to the filled SF2OP FIFO flag            */
#define FLAG_SF_FIFO_OUT_NOT_EMPTY          ((uint32_t)0x00000010)      /*< OP2SF FIFO is not empty                        */
#define FLAG_SF_FIFO_OUT_GREATER            ((uint32_t)0x00000020)      /*< OP2SF FIFO is filled above the specified level */
#define FLAG_SF_FIFO_OUT_FULL               ((uint32_t)0x00000040)      /*< OP2SF FIFO is full                             */
#define FLAG_SF_FIFO_OUT_RD_FROM_EMPTY      ((uint32_t)0x00000080)      /*< Read from the empty OP2SF FIFO flag            */

/*
 * ==========================================================================
 * ------------------------------- UART -------------------------------------
 * ==========================================================================
 */
typedef struct
{
    __IO uint32_t SR;           /*< Status register                    Address offset: 0x00 */
    __IO uint32_t DR;           /*< Data register                      Address offset: 0x04 */
    __IO uint32_t BRR;          /*< Baud rate register                 Address offset: 0x08 */
    __IO uint32_t CR1;          /*< Control register 1                 Address offset: 0x0C */
    __IO uint32_t CR2;          /*< Control register 2                 Address offset: 0x10 */
    __IO uint32_t CR3;          /*< Control register 3                 Address offset: 0x14 */
    __IO uint32_t GTPR;         /*< Guard time and prescaler register  Address offset: 0x18 */
} UART_TypeDef;

/*                                 
 * ==========================================================================
 * ------------------------ UART registers bits -----------------------------
 * ==========================================================================
 */
/*< SR register */
#define UART_SR_CTS             ((uint32_t)0x00000200)      /*< CTS flag                     */
#define UART_SR_LBD             ((uint32_t)0x00000100)      /*< LIN break flag               */
#define UART_SR_TXE             ((uint32_t)0x00000080)      /*< TX empty                     */
#define UART_SR_TC              ((uint32_t)0x00000040)      /*< Transmit complete            */
#define UART_SR_RXNE            ((uint32_t)0x00000020)      /*< Read data register not empty */
#define UART_SR_IDLE            ((uint32_t)0x00000010)      /*< IDLE line detected           */
#define UART_SR_ORE             ((uint32_t)0x00000008)      /*< Overrun error                */
#define UART_SR_NE              ((uint32_t)0x00000004)      /*< Noise error flag             */
#define UART_SR_FE              ((uint32_t)0x00000002)      /*< Frame error                  */
#define UART_SR_PE              ((uint32_t)0x00000001)      /*< Parity error                 */

/*< CR1 register */
#define UART_CR1_UE             ((uint32_t)0x00002000)      /*< USART Enable                 */
#define UART_CR1_M              ((uint32_t)0x00001000)      /*< Byte len. 1 - 9; 0 - 8       */
#define UART_CR1_WAKE           ((uint32_t)0x00000800)      /*< Wake bit                     */
#define UART_CR1_PCE            ((uint32_t)0x00000400)      /*< Parity control enable        */
#define UART_CR1_PS             ((uint32_t)0x00000200)      /*< Parity selection             */
#define UART_CR1_PEIE           ((uint32_t)0x00000100)      /*< PE interrupt enable          */
#define UART_CR1_TXEIE          ((uint32_t)0x00000080)      /*< TXE interrupt enable         */
#define UART_CR1_TCIE           ((uint32_t)0x00000040)      /*< Transmission complete interrupt enable */
#define UART_CR1_RXNEIE         ((uint32_t)0x00000020)      /*< RXNE interrupt enable        */
#define UART_CR1_IDLEIE         ((uint32_t)0x00000010)      /*< IDLE interrupt enable        */
#define UART_CR1_TE             ((uint32_t)0x00000008)      /*< Transmitter enable           */
#define UART_CR1_RE             ((uint32_t)0x00000004)      /*< Receiver enable              */
#define UART_CR1_RWU            ((uint32_t)0x00000002)      /*< Receiver wakeup              */
#define UART_CR1_SBK            ((uint32_t)0x00000001)      /*< Send break                   */

/*< CR2 register */
#define UART_CR2_LINEN          ((uint32_t)0x00004000)      /*< LIN mode enable              */

#define UART_CR2_STOP_MASK      ((uint32_t)0x00003000)      /*< Stop bit type mask           */
#define UART_CR2_STOP_1B        ((uint32_t)0x00000000)      /*< Stop bit type                */
#define UART_CR2_STOP_0_5B      ((uint32_t)0x00001000)      /*< Stop bit type                */
#define UART_CR2_STOP_2B        ((uint32_t)0x00002000)      /*< Stop bit type                */
#define UART_CR2_STOP_1_5B      ((uint32_t)0x00003000)      /*< Stop bit type                */

#define UART_CR2_CLKEN          ((uint32_t)0x00000800)      /*< Clock enable                 */
#define UART_CR2_CPOL           ((uint32_t)0x00000400)      /*< Clock polarity               */
#define UART_CR2_CPHA           ((uint32_t)0x00000200)      /*< Clock phase                  */
#define UART_CR2_LBCL           ((uint32_t)0x00000100)      /*< Last bit clock pulse         */
#define UART_CR2_LBDIE          ((uint32_t)0x00000040)      /*< LIN break detection interrupt enable */
#define UART_CR2_LBDL           ((uint32_t)0x00000020)      /*< LIN break detection length   */
#define UART_CR2_ADD_MASK       ((uint32_t)0x0000000F)      /*< Address of the USART node    */

/*< CR3 register */
#define UART_CR3_CTSIE          ((uint32_t)0x00000400)      /*< CTS interrupt enable   */
#define UART_CR3_CTSE           ((uint32_t)0x00000200)      /*< CTS enable             */
#define UART_CR3_RTSE           ((uint32_t)0x00000100)      /*< RTS enable             */
#define UART_CR3_DMAT           ((uint32_t)0x00000080)      /*< DMA enable transmitter */
#define UART_CR3_DMAR           ((uint32_t)0x00000040)      /*< DMA enable receiver    */
#define UART_CR3_SCEN           ((uint32_t)0x00000020)      /*< Smartcard mode enable  */
#define UART_CR3_NACK           ((uint32_t)0x00000010)      /*< Smartcard NACK         */
#define UART_CR3_HDSEL          ((uint32_t)0x00000008)      /*< Half-duplex selection  */
#define UART_CR3_IRLP           ((uint32_t)0x00000004)      /*< IrDA low-power         */
#define UART_CR3_IREN           ((uint32_t)0x00000002)      /*< IrDA mode enable       */
#define UART_CR3_EIE            ((uint32_t)0x00000001)      /*< Error interrupt enable */

/*
 * ==========================================================================
 * ------------------------------ GPIO --------------------------------------
 * ==========================================================================
 */
typedef struct
{
    __IO uint32_t DATA;             /*< GPIO pins states                                           Address offset: 0x00 */
    __IO uint32_t SET_CLR;          /*< GPIO set/clear pins                                        Address offset: 0x04 */
    __IO uint32_t OD_MODE;          /*< OD mode GPIO pins                                          Address offset: 0x08 */
    __IO uint32_t SLEW_RATE;        /*< Slew rate bus state                                        Address offset: 0x0C */
    __IO uint32_t PORT_MODE;        /*< Port mode selection register                               Address offset: 0x10 */
    __IO uint32_t PU_PD;            /*< Pull up/down selection register                            Address offset: 0x14 */
    __IO uint32_t AF_L;             /*< Alternative function selection register for 0 - 7 pins     Address offset: 0x18 */
    __IO uint32_t AF_H;             /*< Alternative function selection register for 8 - 15 pins    Address offset: 0x1C */
    __IO uint32_t INT_L;            /*< Interrupts modes register for 0 - 7 pins                   Address offset: 0x20 */
    __IO uint32_t INT_H;            /*< Interrupts modes register for 8 - 15 pins                  Address offset: 0x24 */
    __IO uint32_t INT_F;            /*< Interrupts flags register                                  Address offset: 0x28 */
    __IO uint32_t INT_M;            /*< Interrupts masks register                                  Address offset: 0x2C */
    __IO uint32_t FILT_CURR;        /*< Input filter and current excess detector on/off            Address offset: 0x30 */
    __IO uint32_t HCUR;             /*< current excess detector flags register                     Address offset: 0x34 */
    __IO uint32_t LOCK;             /*< Settings lock register                                     Address offset: 0x38 */
} GPIO_TypeDef;

/*
 * ==========================================================================
 * ----------------------- GPIO registers bits ------------------------------
 * ==========================================================================
 */
/*< Settings lock register */
#define GPIO_LOCK 1                 /*< Lock bit */

/*< Port mode selection register posible states */
#define GPIO_MODE_IN                ((uint32_t)0)
#define GPIO_MODE_OUT               ((uint32_t)1)
#define GPIO_MODE_AF                ((uint32_t)2)
#define GPIO_MODE_ANALOG            ((uint32_t)3)

/*< Pull up/down selection register posible states */
#define GPIO_PUPD_NONE              ((uint32_t)0)
#define GPIO_PUPD_UP                ((uint32_t)1)
#define GPIO_PUPD_DOWN              ((uint32_t)2)
#define GPIO_PUPD_RESERVED          ((uint32_t)3)

/*< Slew rate bus posible states */
#define GPIO_SLEWRATE_OFF           ((uint32_t)0)
#define GPIO_SLEWRATE_LOW           ((uint32_t)1)
#define GPIO_SLEWRATE_MED           ((uint32_t)2)
#define GPIO_SLEWRATE_HIGH          ((uint32_t)3)

/*
 * ==========================================================================
 * ------------------- Random Generator registers ---------------------------
 * ==========================================================================
 */
typedef struct
{
    __IO uint32_t STAT_CTRL;        /*< Status and control register                                Address offset: 0x00 */
    __IO uint32_t INT_CTRL;         /*< Interrupt control register                                 Address offset: 0x04 */
    __IO uint32_t CLK_DIV;          /*< Clock divider                                              Address offset: 0x08 */
    __IO uint32_t PAUSE;            /*< Pause register                                             Address offset: 0x0C */
    __IO uint32_t OUTPUT;           /*< Output random                                              Address offset: 0x10 */
    __IO uint32_t PAUSE_CNT;        /*< Pause counter                                              Address offset: 0x14 */
    __IO uint32_t TEMP;             /*< Temp random register                                       Address offset: 0x18 */
} RNG_TypeDef;

/*
 * ==========================================================================
 * ------------------------ RNG registers bits ------------------------------
 * ==========================================================================
 */
/*< Status and control register */
#define RNG_FORCE_EN                ((uint32_t)0x00000001)
#define RNG_EN_STATE                ((uint32_t)0x00000002)
#define RNG_BUSY                    ((uint32_t)0x00000004)
#define RNG_UNREAD_DATA             ((uint32_t)0x00000008)

#define RNG_GEN_MASK                ((uint32_t)0x000000F0)
#define RNG_GEN_0                   ((uint32_t)0x00000010)
#define RNG_GEN_1                   ((uint32_t)0x00000020)
#define RNG_GEN_2                   ((uint32_t)0x00000040)
#define RNG_GEN_3                   ((uint32_t)0x00000080)

#define RNG_WORK_MODE_MASK          ((uint32_t)0x00000300)
#define RNG_WORK_MODE_STOPPED       ((uint32_t)0x00000000)
#define RNG_WORK_MODE_ONCE          ((uint32_t)0x00000100)
#define RNG_WORK_MODE_AFTER_READ    ((uint32_t)0x00000200)
#define RNG_WORK_MODE_ALWAYS        ((uint32_t)0x00000300)

#define RNG_BIT_CN_MASK             ((uint32_t)0x0003F000)

/*< Status and control register */
#define RNG_INT_EN                  ((uint32_t)0x00000001)
#define RNG_INT_UNREAD_DATA         ((uint32_t)0x00000002)

/*
 * ==========================================================================
 * --------------------- Noise Generator registers --------------------------
 * ==========================================================================
 */
typedef struct
{
    __IO uint32_t CTRL;             /*< Control register                       Address offset: 0x00 */
    __IO uint32_t SEED[4];          /*< Seed for generators                    Address offset: 0x04 */
    __IO uint32_t LFSR[4];          /*< Current generators internal states     Address offset: 0x14 */
    __IO uint32_t RAND[4];          /*< Random output values of generators     Address offset: 0x24 */
    __IO uint32_t POLY[4];          /*< TAP for generators                     Address offset: 0x34 */ 
    __IO uint32_t PERIODS;          /*< Generators output update period        Address offset: 0x44 */
    __IO uint32_t MASK[4];          /*< Output mask for generators             Address offset: 0x48 */
} NOISE_TypeDef;

/*
 * ==========================================================================
 * ------------------- Noise Generator registers bits -----------------------
 * ==========================================================================
 */
/*< Status register bits */
#define NOISE_MODE_0            ((uint32_t)0x00000001U)     /*< Zero PRNG mode. 0 - init; 1 - work                                    */
#define NOISE_MODE_1            ((uint32_t)0x00000002U)     /*< First PRNG mode. 0 - init; 1 - work                                   */
#define NOISE_MODE_2            ((uint32_t)0x00000004U)     /*< Second PRNG mode. 0 - init; 1 - work                                  */
#define NOISE_MODE_3            ((uint32_t)0x00000008U)     /*< Third PRNG mode. 0 - init; 1 - work                                   */
#define NOISE_ALLOW_OUTPUT_0    ((uint32_t)0x00000010U)     /*< Zero PRNG output. 0 - forbidden (except direct output); 1 - allowed   */
#define NOISE_ALLOW_OUTPUT_1    ((uint32_t)0x00000020U)     /*< First PRNG output. 0 - forbidden (except direct output); 1 - allowed  */
#define NOISE_ALLOW_OUTPUT_2    ((uint32_t)0x00000040U)     /*< Second PRNG output. 0 - forbidden (except direct output); 1 - allowed */
#define NOISE_ALLOW_OUTPUT_3    ((uint32_t)0x00000080U)     /*< Third PRNG output. 0 - forbidden (except direct output); 1 - allowed  */
                                                            /*< reserved                                                              */
#define NOISE_CASCADE_1         ((uint32_t)0x00000200U)     /*< First PRNG cascade mode. 0 - off; 1 - on                              */
#define NOISE_CASCADE_2         ((uint32_t)0x00000400U)     /*< Second PRNG cascade mode. 0 - off; 1 - on                             */
#define NOISE_CASCADE_3         ((uint32_t)0x00000800U)     /*< Third PRNG cascade mode. 0 - off; 1 - on                              */
#define NOISE_DIRECT_OUTPUT_0   ((uint32_t)0x00001000U)     /*< Zero PRNG direct output. 0 - off; 1 - on                              */
#define NOISE_DIRECT_OUTPUT_1   ((uint32_t)0x00002000U)     /*< First PRNG direct output. 0 - off; 1 - on                             */
#define NOISE_DIRECT_OUTPUT_2   ((uint32_t)0x00004000U)     /*< Second PRNG direct output. 0 - off; 1 - on                            */
#define NOISE_DIRECT_OUTPUT_3   ((uint32_t)0x00008000U)     /*< Third PRNG direct output. 0 - off; 1 - on                             */

/*
 * ==========================================================================
 * -------------------------- System registers ------------------------------
 * ==========================================================================
 */
typedef struct
{
  __IO uint32_t STATUS;           /*< Status register                       Address offset: 0x00 */
  __IO uint32_t CTRL;             /*< Control register                      Address offset: 0x04 */
  __IO uint32_t PEN;              /*< Peripheral enable register            Address offset: 0x08 */
  __IO uint32_t SEN;              /*< Sleep enabled peripherals             Address offset: 0x0C */
  __IO uint32_t DSEN;             /*< Deep sleep enabled peripherals        Address offset: 0x10 */
  __IO uint32_t MMAP;             /*< Memory map register                   Address offset: 0x14 */
  __IO uint32_t KSEED;            /*< Key memory scrambler seed register    Address offset: 0x18 */
  __IO uint32_t RSEED;            /*< RAM scrambler seed register           Address offset: 0x1C */
  __IO uint32_t M_SCTRL;          /*< Memory special control,               Address offset: 0x20 */
  __IO uint32_t PLL_CTRL;         /*< PLL control,                          Address offset: 0x24 */
  __IO uint32_t HSI_CTRL;         /*< HSI generator control,                Address offset: 0x28 */
} SYS_TypeDef;

/*
 * ==========================================================================
 * ----------------------- System registers bits ----------------------------
 * ==========================================================================
 */
/*< Status register bits */
#define SYS_ST_MEM_CPT            ((uint32_t)0x00000001U)     /*< Memory corruption detected */
#define SYS_ST_KEY_CPT            ((uint32_t)0x00000002U)     /*< Key memory corruption detected */
#define SYS_ST_OTP_CPT            ((uint32_t)0x00000004U)     /*< OTP test options corruption detected */
#define SYS_ST_RAM_CPT            ((uint32_t)0x00000008U)     /*< RAM corruption detected */
#define SYS_ST_KEY_NRST           ((uint32_t)0x00000010U)     /*< Key memory not reset */
                                                              /*< 5  - reserved */    
#define SYS_ST_SENS_ALARM         ((uint32_t)0x00000040U)     /*< Sensor alarm state */
#define SYS_ST_DEBUG_ST           ((uint32_t)0x00000080U)     /*< Debug enabled state */
#define SYS_ST_MCTRL_LOCK_ST      ((uint32_t)0x00000100U)     /*< Memory special control lock state */
                                                              /*< 9  - reserved */    
#define SYS_ST_DEBUG_POW          ((uint32_t)0x00000400U)     /*< Debug power state */    
                                                              /*< 11 - reserved */    
                                                              /*< 12 - reserved */    
#define SYS_ST_PLL_RDY            ((uint32_t)0x00002000U)     /*< PLL ready */    
                                                              /*< 13 - reserved */    
                                                              /*< 14 - reserved */  
                                                              /*< 15 - reserved */    
                                                              /*< 16 - reserved */  

/*< Control register bits */                                 
                                                              /*< 0  - reserved */
#define SYS_CTRL_KEY_TEST         ((uint32_t)0x00000002U)     /*< Enable key memory test */
#define SYS_CTRL_RAM_TEST         ((uint32_t)0x00000004U)     /*< Enable RAM test */
#define SYS_CTRL_OTP_TEST         ((uint32_t)0x00000008U)     /*< Enable OTP test */
#define SYS_CTRL_KEY_RST          ((uint32_t)0x00000010U)     /*< Reset key memory */
#define SYS_CTRL_PCLK_HALF        ((uint32_t)0x00000020U)     /*< Half frequency APB clock mode  */
                                                              /*< 6  - reserved */
#define SYS_CTRL_DEBUG_EN         ((uint32_t)0x00000080U)     /*< Debug enable */
                                                              /*< 8  - reserved */
#define SYS_CTRL_DPRG_START       ((uint32_t)0x00000200U)     /*< Debug start program mode */
                                                              /*< 10 - reserved */
#define SYS_CTRL_CPU_LDONE        ((uint32_t)0x00000800U)     /*< CPU load done signal */    
#define SYS_CTRL_PLL_EN           ((uint32_t)0x00001000U)     /*< PLL enable */    
                                                              /*< 13 - reserved */    
#define SYS_CTRL_PLL_OUTM         ((uint32_t)0x00004000U)     /*< PLL out mode */
#define SYS_CTRL_CLK_SRC          ((uint32_t)0x00008000U)     /*< CLK source */
#define SYS_CTRL_OCLK_D0          ((uint32_t)0x00010000U)     /*< CLK to out Divider bit 0 */
#define SYS_CTRL_OCLK_D1          ((uint32_t)0x00020000U)     /*< CLK to out Divider bit 1 */

/*< Peripheral enable register */                            
#define SYS_PEN_GATE_0            ((uint32_t)0x00000001U)     /*< Gate Channel 0 enable */
#define SYS_PEN_GATE_1            ((uint32_t)0x00000002U)     /*< Gate Channel 1 enable */
#define SYS_PEN_ALU               ((uint32_t)0x00000004U)     /*< ALU enable */
#define SYS_PEN_LBLOCK            ((uint32_t)0x00000008U)     /*< L Block enable */
#define SYS_PEN_SBLOCK0           ((uint32_t)0x00000010U)     /*< S Block 0 enable */
#define SYS_PEN_SBLOCK1           ((uint32_t)0x00000020U)     /*< S Block 1 enable */
#define SYS_PEN_SBLOCK2           ((uint32_t)0x00000040U)     /*< S Block 2 enable */
#define SYS_PEN_SBLOCK3           ((uint32_t)0x00000080U)     /*< S Block 3 enable */
#define SYS_PEN_SBLOCK4           ((uint32_t)0x00000100U)     /*< S Block 4 enable */
#define SYS_PEN_SBLOCK5           ((uint32_t)0x00000200U)     /*< S Block 5 enable */
#define SYS_PEN_SBLOCK6           ((uint32_t)0x00000400U)     /*< S Block 6 enable */
#define SYS_PEN_SBLOCK7           ((uint32_t)0x00000800U)     /*< S Block 7 enable */
#define SYS_PEN_PBYTE             ((uint32_t)0x00001000U)     /*< P Byte enable */
#define SYS_PEN_PBIT0             ((uint32_t)0x00002000U)     /*< P Bit 0 enable */
#define SYS_PEN_PBIT1             ((uint32_t)0x00004000U)     /*< P Bit 1 enable */
#define SYS_PEN_PBIT2             ((uint32_t)0x00008000U)     /*< P Bit 2 enable */
#define SYS_PEN_PBIT3             ((uint32_t)0x00010000U)     /*< P Bit 3 enable */
#define SYS_PEN_DES               ((uint32_t)0x00020000U)     /*< DES enable */
#define SYS_PEN_GPIO              ((uint32_t)0x00040000U)     /*< GPIO enable */
#define SYS_PEN_NOISE             ((uint32_t)0x00080000U)     /*< Noise Generator enable */
#define SYS_PEN_UART              ((uint32_t)0x00100000U)     /*< UART enable */
#define SYS_PEN_SENSORS           ((uint32_t)0x00200000U)     /*< Sensors enable */
#define SYS_PEN_RANDOM            ((uint32_t)0x00400000U)     /*< Random enable */
#define SYS_PEN_ALL               ((uint32_t)0x007FFFFFU)     /*< Enable all */

/*< Sleep mode enabled peripherals */
#define SYS_SEN_GATE_0            ((uint32_t)0x00000001U)     /*< Gate Channel 0 enable */
#define SYS_SEN_GATE_1            ((uint32_t)0x00000002U)     /*< Gate Channel 1 enable */
#define SYS_SEN_ALU               ((uint32_t)0x00000004U)     /*< ALU enable */
#define SYS_SEN_LBLOCK            ((uint32_t)0x00000008U)     /*< L Block enable */
#define SYS_SEN_SBLOCK0           ((uint32_t)0x00000010U)     /*< S Block 0 enable */
#define SYS_SEN_SBLOCK1           ((uint32_t)0x00000020U)     /*< S Block 1 enable */
#define SYS_SEN_SBLOCK2           ((uint32_t)0x00000040U)     /*< S Block 2 enable */
#define SYS_SEN_SBLOCK3           ((uint32_t)0x00000080U)     /*< S Block 3 enable */
#define SYS_SEN_SBLOCK4           ((uint32_t)0x00000100U)     /*< S Block 4 enable */
#define SYS_SEN_SBLOCK5           ((uint32_t)0x00000200U)     /*< S Block 5 enable */
#define SYS_SEN_SBLOCK6           ((uint32_t)0x00000400U)     /*< S Block 6 enable */
#define SYS_SEN_SBLOCK7           ((uint32_t)0x00000800U)     /*< S Block 7 enable */
#define SYS_SEN_PBYTE             ((uint32_t)0x00001000U)     /*< P Byte enable */
#define SYS_SEN_PBIT0             ((uint32_t)0x00002000U)     /*< P Bit 0 enable */
#define SYS_SEN_PBIT1             ((uint32_t)0x00004000U)     /*< P Bit 1 enable */
#define SYS_SEN_PBIT2             ((uint32_t)0x00008000U)     /*< P Bit 2 enable */
#define SYS_SEN_PBIT3             ((uint32_t)0x00010000U)     /*< P Bit 3 enable */
#define SYS_SEN_DES               ((uint32_t)0x00020000U)     /*< DES enable */
#define SYS_SEN_GPIO              ((uint32_t)0x00040000U)     /*< GPIO enable */
#define SYS_SEN_NOISE             ((uint32_t)0x00080000U)     /*< Noise Generator enable */
#define SYS_SEN_UART              ((uint32_t)0x00100000U)     /*< UART enable */
#define SYS_SEN_SENSORS           ((uint32_t)0x00200000U)     /*< Sensors enable */
#define SYS_SEN_RANDOM            ((uint32_t)0x00400000U)     /*< Random enable */

/*< Deep sleep mode enabled peripherals  */
#define SYS_DSEN_GATE_0           ((uint32_t)0x00000001U)     /*< Gate Channel 0 enable */
#define SYS_DSEN_GATE_1           ((uint32_t)0x00000002U)     /*< Gate Channel 1 enable */
#define SYS_DSEN_ALU              ((uint32_t)0x00000004U)     /*< ALU enable */
#define SYS_DSEN_LBLOCK           ((uint32_t)0x00000008U)     /*< L Block enable */
#define SYS_DSEN_SBLOCK0          ((uint32_t)0x00000010U)     /*< S Block 0 enable */
#define SYS_DSEN_SBLOCK1          ((uint32_t)0x00000020U)     /*< S Block 1 enable */
#define SYS_DSEN_SBLOCK2          ((uint32_t)0x00000040U)     /*< S Block 2 enable */
#define SYS_DSEN_SBLOCK3          ((uint32_t)0x00000080U)     /*< S Block 3 enable */
#define SYS_DSEN_SBLOCK4          ((uint32_t)0x00000100U)     /*< S Block 4 enable */
#define SYS_DSEN_SBLOCK5          ((uint32_t)0x00000200U)     /*< S Block 5 enable */
#define SYS_DSEN_SBLOCK6          ((uint32_t)0x00000400U)     /*< S Block 6 enable */
#define SYS_DSEN_SBLOCK7          ((uint32_t)0x00000800U)     /*< S Block 7 enable */
#define SYS_DSEN_PBYTE            ((uint32_t)0x00001000U)     /*< P Byte enable */
#define SYS_DSEN_PBIT0            ((uint32_t)0x00002000U)     /*< P Bit 0 enable */
#define SYS_DSEN_PBIT1            ((uint32_t)0x00004000U)     /*< P Bit 1 enable */
#define SYS_DSEN_PBIT2            ((uint32_t)0x00008000U)     /*< P Bit 2 enable */
#define SYS_DSEN_PBIT3            ((uint32_t)0x00010000U)     /*< P Bit 3 enable */
#define SYS_DSEN_DES              ((uint32_t)0x00020000U)     /*< DES enable */
#define SYS_DSEN_GPIO             ((uint32_t)0x00040000U)     /*< GPIO enable */
#define SYS_DSEN_NOISE            ((uint32_t)0x00080000U)     /*< Noise Generator enable */
#define SYS_DSEN_UART             ((uint32_t)0x00100000U)     /*< UART enable */
#define SYS_DSEN_SENSORS          ((uint32_t)0x00200000U)     /*< Sensors enable */
#define SYS_DSEN_RANDOM           ((uint32_t)0x00400000U)     /*< Random enable */

/*< Memory map register */                                   
#define SYS_MMAP_RAM              ((uint32_t)0x00000001U)     /*< Map RAM to zero address */

/*< PLL control */
#define SYS_PLL_MULT_MASK         ((uint32_t)0x000001FFU)     /*< PLL multiplier mask */
#define SYS_PLL_MULT_SHIFT        ((uint32_t)00U)             /*< PLL multiplier shift */

#define SYS_PLL_DIV_MASK          ((uint32_t)0x00FF0000U)     /*< PLL divider mask */
#define SYS_PLL_DIV_SHIFT         ((uint32_t)16U)             /*< PLL divider shift */

#define SYS_PLL_PDIV_MASK         ((uint32_t)0x03000000U)     /*< PLL power divider mask */
#define SYS_PLL_PDIV_SHIFT        ((uint32_t)24U)             /*< PLL power divider shift */

/*
 * ==========================================================================
 * ------------------------ Peripherals memory map --------------------------
 * ==========================================================================
 */
#define ROM_BASE              ((uint32_t)0x10000000U)               /*< ROM base address in the alias region */
#define OTP_BASE              ((uint32_t)0x10100000U)               /*< Special FLASH base address in the alias region */
#define RAM_BASE              ((uint32_t)0x20000000U)               /*< RAM base address in the alias region */
#define KEYRAM_BASE           ((uint32_t)0x20100000U)               /*< Key RAM base address in the alias region */

#define GATE0_BASE            ((uint32_t)0x40000000U)               /*< Gate 1 base address in the alias region */
#define GATE1_BASE            ((uint32_t)0x40000080U)               /*< Gate 2 base address in the alias region */
#define AHBPERIPH_BASE        ((uint32_t)0x40100000U)               /*< AHB Peripheral base address in the alias region */
#define SYS_BASE              ((uint32_t)0x40200000U)               /*< System registers base address in the alias region */
#define APBPERIPH_BASE        ((uint32_t)0x40400000U)               /*< APB Peripheral base address in the alias region */

/*< AHB peripherals */
#define ALU_BASE              (AHBPERIPH_BASE)
#define ALU_MBASE             (ALU_BASE + 0x00001000)               /*< ALU Prpogramm memory base address */
#define ALU_RBASE             (ALU_BASE + 0x00002000)               /*< ALU Long registers base address */

#define ALU_RA_BASE           (ALU_RBASE)                           /*< ALU Long registers */
#define ALU_RB_BASE           (ALU_RBASE + 0x00000800)
#define ALU_RC_BASE           (ALU_RBASE + 0x00001000)
#define ALU_RD_BASE           (ALU_RBASE + 0x00001800)
#define ALU_RE_BASE           (ALU_RBASE + 0x00002000)
#define ALU_RF_BASE           (ALU_RBASE + 0x00002800)
#define ALU_RU_BASE           (ALU_RBASE + 0x00003000)

#define OTP_MEM_BASE          (OTP_BASE)
#define OTP_REGS_BASE         (OTP_BASE + 0x00010000)

#define LBLOCK_BASE           (AHBPERIPH_BASE + 0x00004000)         /*< L Block base address */

#define SBLOCK0_BASE          (AHBPERIPH_BASE + 0x00008000)         /*< S Blocks base addresses */
#define SBLOCK1_BASE          (AHBPERIPH_BASE + 0x0000C000)
#define SBLOCK2_BASE          (AHBPERIPH_BASE + 0x00010000)
#define SBLOCK3_BASE          (AHBPERIPH_BASE + 0x00014000)
#define SBLOCK4_BASE          (AHBPERIPH_BASE + 0x00018000)
#define SBLOCK5_BASE          (AHBPERIPH_BASE + 0x0001C000)
#define SBLOCK6_BASE          (AHBPERIPH_BASE + 0x00020000)
#define SBLOCK7_BASE          (AHBPERIPH_BASE + 0x00024000)
    
#define PBYTE_BASE            (AHBPERIPH_BASE + 0x00028000)         /*< P Byte base address */
    
#define PBIT0_BASE            (AHBPERIPH_BASE + 0x0002C000)         /*< P Bits base addresses */
#define PBIT1_BASE            (AHBPERIPH_BASE + 0x00030000)   
#define PBIT2_BASE            (AHBPERIPH_BASE + 0x00034000)   
#define PBIT3_BASE            (AHBPERIPH_BASE + 0x00038000)   
    
#define DES_BASE              (AHBPERIPH_BASE + 0x0003C000)         /*< Des base address */

/*< APB peripherals */
#define NOISE_GEN_BASE        (APBPERIPH_BASE)                      /*< Noise Generator base address */
#define UART_BASE             (APBPERIPH_BASE + 0x00004000)         /*< UART base address */
#define SENSORS_BASE          (APBPERIPH_BASE + 0x00008000)         /*< Sensors base address */
#define RANDOM_BASE           (APBPERIPH_BASE + 0x0000C000)         /*< Random sensor base address */
#define GPIO_BASE             (AHBPERIPH_BASE + 0x00040000)         /*< GPIO base address */

/*
 * ==========================================================================
 * ----------------------- Peripherals declaration --------------------------
 * ==========================================================================
 */
#define ROM                 ((uint32_t *)ROM_BASE)                  /*< ROM memory */
#define RAM                 ((uint32_t *)RAM_BASE)                  /*< RAM memory */
#define KEY_RAM             ((uint32_t *)KEYRAM_BASE)               /*< RAM for keys memory */
#define OTP                 ((OTP_TypeDef *)OTP_REGS_BASE)          /*< OTP registers */
#define OTP_MEM             ((uint32_t *)OTP_MEM_BASE)              /*< OTP memory */

#define ROM_SZ              ((uint32_t)0x00002000)
#define RAM_SZ              ((uint32_t)0x00020000)
#define KEY_RAM_SZ          ((uint32_t)0x00000800)
#define OTP_MEM_SZ          ((uint32_t)0x00010000)

#define SYS                 ((SYS_TypeDef *) SYS_BASE)              /*< System registers */

#define ALU                 ((ALU_TypeDef *) ALU_BASE)              /*< ALU registers */
#define ALU_MEM             ((uint32_t *) ALU_MBASE)                /*< ALU memory*/

#define ALU_RA              ((uint32_t *) ALU_RA_BASE)              /*< ALU Long registers */
#define ALU_RB              ((uint32_t *) ALU_RB_BASE)
#define ALU_RC              ((uint32_t *) ALU_RC_BASE)
#define ALU_RD              ((uint32_t *) ALU_RD_BASE)
#define ALU_RE              ((uint32_t *) ALU_RE_BASE)
#define ALU_RF              ((uint32_t *) ALU_RF_BASE)
#define ALU_RU              ((uint32_t *) ALU_RU_BASE)

#define GATE0_REGS          ((GATE_TypeDef *)GATE0_BASE)            /*< Gate 1 registers */
#define GATE1_REGS          ((GATE_TypeDef *)GATE1_BASE)            /*< Gate 2 registers */

#define LBLOCK              ((LBLOCK_TypeDef *) LBLOCK_BASE)        /*< L Block registers */

#define SBLOCK0             ((SBLOCK_TypeDef *) SBLOCK0_BASE)       /*< S Blocks registers  */
#define SBLOCK1             ((SBLOCK_TypeDef *) SBLOCK1_BASE)
#define SBLOCK2             ((SBLOCK_TypeDef *) SBLOCK2_BASE)
#define SBLOCK3             ((SBLOCK_TypeDef *) SBLOCK3_BASE)
#define SBLOCK4             ((SBLOCK_TypeDef *) SBLOCK4_BASE)
#define SBLOCK5             ((SBLOCK_TypeDef *) SBLOCK5_BASE)
#define SBLOCK6             ((SBLOCK_TypeDef *) SBLOCK6_BASE)
#define SBLOCK7             ((SBLOCK_TypeDef *) SBLOCK7_BASE)

#define PBYTE               ((PBYTE_TypeDef *) PBYTE_BASE)          /*< P Byte registers */

#define PBIT0               ((PBIT_TypeDef *) PBIT0_BASE)           /*< P Bits registers */
#define PBIT1               ((PBIT_TypeDef *) PBIT1_BASE)
#define PBIT2               ((PBIT_TypeDef *) PBIT2_BASE)
#define PBIT3               ((PBIT_TypeDef *) PBIT3_BASE)

#define DES                 ((DES_TypeDef *) DES_BASE)              /*< Des registers */

#define NOISE_GEN           ((NOISE_TypeDef *) NOISE_GEN_BASE)      /*< Noise Generator registers */
#define UART                ((UART_TypeDef *) UART_BASE)            /*< UART registers */
#define RNG                 ((RNG_TypeDef *) RANDOM_BASE)           /*< Random generator registers */
#define GPIO                ((GPIO_TypeDef *) GPIO_BASE)            /*< GPIO registers */

#endif // CRYPTO_CM0_HEADER
