/**
* @file noise.h
* @brief Noise generator header
* @author Kosenkov
* @date 18.04.2018
*/

#ifndef __NOISE_H__
#define __NOISE_H__

#include "types.h"

#define NOISE_RAND_SZ       ((uint8_t)4)                    /*< Size of noise random */

#define NOISE_CASCADE_OFF   ((uint8_t)0)                    /*< Cascade mode off */
#define NOISE_CASCADE_ON    ((uint8_t)!NOISE_CASCADE_OFF)   /*< Cascade mode on  */

#define NOISE_OUTPUT_OFF    ((uint8_t)0)                    /*< Cascade mode off */
#define NOISE_OUTPUT_ON     ((uint8_t)!NOISE_CASCADE_OFF)   /*< Cascade mode on  */

/**
* Noise module initialization
*
* @param output Random output mode on/off
* @param cascade Cascade mode on/off
* @retval CM0_OK if success
* @retval CM0_NOISE_ERR otherwise
* @author Kosenkov
* @date 18.05.2018
*/
CM0_STATUS noiseInit(uint8_t cascade, uint8_t output);

/**
* Get random number from noise module
*
* @param rnd Array for randoms
* @param sz Size of array
* @author Kosenkov
* @date 18.05.2018
*/
void noiseRandom(uint32_t* rnd, Sz32 sz);

/**
* Runs random quality tests using noise generator
* 
* @retval CM0_OK if success
* @retval CM0_RNG_TEST_ERR otherwise
* @author Kosenkov
* @date 25.05.2018
*/
CM0_STATUS noiseTest(void);

#endif
