/**
* @file rng.h
* @brief Random generator header
* @author Kosenkov
* @date 18.04.2018
*/

#ifndef __RNG_H__
#define __RNG_H__

#include "types.h"

/**
* Sets to random generator current core clock
* It is important for proper RNG working
*
* @param clock Current core clock
* @author Kosenkov
* @date 18.05.2018
*/
void rngSetClock(uint32_t clock);

/**
* Sets random generator mode
* @see cm0.h
*
* @param mode Work mode
* @author Kosenkov
* @date 18.05.2018
*/
void rngSetMode(uint32_t mode);

/**
* Gets random number
*
* @param word Address for random number store to
* @retval CM0_OK if success
* @retval CM0_FAIL otherwise
* @author Kosenkov
* @date 18.05.2018
*/
CM0_STATUS rngGetWord(uint32_t* word);

/**
* Runs random quality tests
* 
* @param clock Current core clock
* @retval CM0_OK if success
* @retval CM0_RNG_TEST_ERR otherwise
* @author Kosenkov
* @date 23.05.2018
*/
CM0_STATUS rngTest(uint32_t clock);

#endif
